import os
from glob import glob
from itertools import chain
from tempfile import TemporaryFile
from zipfile import ZipFile

from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
from tqdm import tqdm

from tronclass_cli.api import Api
from tronclass_cli.command import Command
from tronclass_cli.middleware.api import ApiMiddleware


def zip_files(fp, paths):
    with ZipFile(fp, 'w') as zf:
        for path in paths:
            path = os.path.normpath(path)
            if os.path.isdir(path):
                for dirpath, dirnames, filenames in os.walk(path):
                    for name in sorted(dirnames):
                        path = os.path.normpath(os.path.join(dirpath, name))
                        zf.write(path, path)
                    for name in filenames:
                        path = os.path.normpath(os.path.join(dirpath, name))
                        if os.path.isfile(path):
                            zf.write(path, path)
            else:
                zf.write(path)


def post_file(api: Api, name, fp):
    fp.seek(0, 2)
    file_size = fp.tell()
    fp.seek(0, 0)

    upload = api.post_uploads(name, file_size)

    progress_bar = None

    def update_callback(monitor):
        progress_bar.n = monitor.bytes_read
        progress_bar.refresh()

    data = MultipartEncoder(
        fields={'file': (name, fp)}
    )
    monitor = MultipartEncoderMonitor(data, update_callback)
    progress_bar = tqdm(desc=name, total=data.len, unit='iB', unit_scale=True)
    headers = {'Content-Type': monitor.content_type, 'Content-Length': str(data.len)}
    res = api.session.put(upload['upload_url'], data=monitor, headers=headers)

    if progress_bar.n != data.len:
        raise IOError('file size mismatches')
    progress_bar.close()
    res.raise_for_status()
    return upload['id']


class HomeworkSubmitCommand(Command):
    name = 'courses.submit'
    middleware_classes = [ApiMiddleware]

    def _init_parser(self):
        self._parser.add_argument('activity_id', help='homework activity id')
        self._parser.add_argument('paths', nargs='+', type=glob,
                                  help='files or directories to be submitted, glob supported')
        self._parser.add_argument('--compress', metavar='NAME', help='compress the files with given file name')
        self._parser.add_argument('--draft', action='store_true', help='submit as a draft')

    def _exec(self, args):
        self._ctx.api.get_activity(args.activity_id)
        paths = chain(*args.paths)
        uploads = []
        if args.compress is not None:
            with TemporaryFile() as fp:
                zip_files(fp, paths)
                uploads.append(post_file(self._ctx.api, args.compress, fp))
        else:
            for path in paths:
                if not os.path.isfile(path):
                    raise TypeError('folders are not supported to be uploaded, use --compress to create a zipped file')
                with open(path, 'rb') as fp:
                    uploads.append(post_file(self._ctx.api, os.path.basename(path), fp))
        self._ctx.api.post_submissions(args.activity_id, uploads, is_draft=args.draft)



import shutil

from tronclass_cli.command import Command
from tronclass_cli.middleware.cache import CacheMiddleware


class CacheCommand(Command):
    name = 'cache'

    def _exec(self, args):
        self._parser.print_help()


class CacheClearCommand(Command):
    name = 'cache.clear'
    middleware_classes = [CacheMiddleware]

    def _init_parser(self):
        pass

    def _exec(self, args):
        shutil.rmtree(args.cache_dir)



from tronclass_cli.command import Command

from .list import ActivitiesListCommand
from .view import ActivitiesViewCommand
from .download import ActivitiesDownloadCommand


class ActivitiesCommand(Command):
    name = 'activities'

    def _exec(self, args):
        self._parser.print_help()



import re

from dateutil.parser import isoparse
from tabulate import tabulate

from tronclass_cli.middleware import Middleware
from tronclass_cli.utils import flatten_dict, dict_select


class TableMiddleware(Middleware):
    name = 'table'

    def _init_parser(self):
        group = self._parser.add_argument_group(self.name)
        group.add_argument('--table-fmt', default='simple', help='table format')
        group.add_argument('--date-fmt', default='%c', help='date format')

    def _exec(self, args):
        date_regex = re.compile(r'\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d(\.\d+)?([+-][0-2]\d:[0-5]\d|Z)')

        def process_value(value):
            if isinstance(value, list):
                return ', '.join([str(x) for x in value])
            elif isinstance(value, str) and date_regex.search(value):
                return isoparse(value).astimezone().strftime(args.date_fmt)
            return value

        def process_dict(data, fields):
            data = flatten_dict(data)
            data = dict_select(data, fields)
            return {k: process_value(v) for k, v in data.items()}

        def print_table(data, fields):
            data = [process_dict(d, fields) for d in data]
            print(tabulate(data, headers='keys', tablefmt=args.table_fmt))

        self._ctx.print_table = print_table



from argparse import ArgumentParser

from tronclass_cli.command.activities.view import ActivitiesViewCommand
from tronclass_cli.command.cache import CacheCommand, CacheClearCommand
from tronclass_cli.command.root import RootCommand
from tronclass_cli.command.todo import TodoCommand
from tronclass_cli.command.courses import CoursesCommand, CoursesListCommand
from tronclass_cli.command.homework import HomeworkCommand, HomeworkListCommand, HomeworkSubmitCommand
from tronclass_cli.command.activities import ActivitiesCommand, ActivitiesListCommand, ActivitiesDownloadCommand
from tronclass_cli.middleware import Context
from tronclass_cli.utils import interact

parser = ArgumentParser()
root_command = RootCommand(parser, Context())
root_command.add_sub_command('todo', TodoCommand, aliases=['t', 'td'], help='view to-do list')

courses_command = root_command.add_sub_command('courses', CoursesCommand, aliases=['c'], help='manage all courses')
courses_command.add_sub_command('list', CoursesListCommand, aliases=['l', 'ls'], help='list courses')

activities_command = root_command.add_sub_command('activities', ActivitiesCommand, aliases=['a'],
                                                  help='manage activities of the courses')
activities_command.add_sub_command('list', ActivitiesListCommand, aliases=['l', 'ls'], help='list activities')
activities_command.add_sub_command('view', ActivitiesViewCommand, aliases=['v'], help='view an activity')
activities_command.add_sub_command('download', ActivitiesDownloadCommand, aliases=['d', 'dl'],
                                   help='download attached files')

homework_command = root_command.add_sub_command('homework', HomeworkCommand, aliases=['h', 'hw'],
                                                help='manage homework of the courses')
homework_command.add_sub_command('list', HomeworkListCommand, aliases=['l', 'ls'], help='list homework')
homework_command.add_sub_command('submit', HomeworkSubmitCommand, aliases=['s'], help='submit homework')

cache_command = root_command.add_sub_command('cache', CacheCommand, help='manage the cache data')
cache_command.add_sub_command('clear', CacheClearCommand, help='clear the cache data')


def main():
    args = root_command.parse_args()
    try:
        args.__middleware.exec(args)
        args.__middleware.dispose()
    except Exception as ex:
        interact.error(f'fatal: {ex}')


if __name__ == '__main__':
    main()



class AuthProvider:
    desc = None

    def __init__(self):
        pass

    def login(self, *args, **kwargs):
        pass


class AuthError(Exception):
    def __init__(self, message='Failed to authenticate. Please check your user name and password.'):
        super().__init__(message)



import sys


def prompt(text: str):
    print(text)


def prompt_input(prompt: str, default: str = None):
    if default is not None:
        prompt += f' [{default}]'
    prompt += ': '
    res = input(prompt)
    if res == '':
        res = default
    return res


def error(prompt):
    print(prompt, file=sys.stderr)


def select(prompt: str, options):
    print(prompt)
    for i, option in enumerate(options):
        print(f'[{i}] {option}')
    while True:
        try:
            i = int(input(f'[0-{len(options) - 1}]: '))
            if i < 0 or i >= len(options):
                raise ValueError()
            return i
        except ValueError:
            error(f'Please input an integer between 0 and {len(options) - 1}.')
            continue



from pathlib import Path

from tronclass_cli.middleware import Middleware
from tronclass_cli.utils.cache import Cache


class CacheMiddleware(Middleware):
    name = 'cache'

    def _init_parser(self):
        group = self._parser.add_argument_group(self.name)
        default_dir = self.default_root / 'dat'
        group.add_argument('--cache-dir', type=Path, default=default_dir,
                           help=f'path to the cache directory, default: {default_dir}')
        group.add_argument('--force-update', action='store_true',
                           help='fetch the data from the server and update the cache')

    def _exec(self, args):
        args.cache_dir.mkdir(0o700, True, True)
        if 'cache' not in self._ctx:
            self._ctx.cache = Cache(args.cache_dir / 'cache', args.force_update)

    def _dispose(self):
        self._ctx.cache.close()



from tronclass_cli.api.auth.zjuam import ZjuamAuthProvider

_auth_providers = {'zju': ZjuamAuthProvider}


def get_auth_provider(name):
    return _auth_providers[name]


def get_all_auth_providers():
    return _auth_providers



from tronclass_cli.command import Command

from .list import HomeworkListCommand
from .submit import HomeworkSubmitCommand


class HomeworkCommand(Command):
    name = 'courses'

    def _exec(self, args):
        self._parser.print_help()



import requests
import rsa
from bs4 import BeautifulSoup

from tronclass_cli.api.auth import AuthProvider, AuthError

PUB_KEY_URL = 'https://zjuam.zju.edu.cn/cas/v2/getPubKey'
LOGIN_URL = 'https://zjuam.zju.edu.cn/cas/login'


def rsa_encrypt(message, pub_key):
    key_length = rsa.common.byte_size(pub_key.n)
    payload = rsa.transform.bytes2int(message)
    cypher = rsa.core.encrypt_int(payload, pub_key.e, pub_key.n)
    return rsa.transform.int2bytes(cypher, key_length)


class ZjuamAuthProvider(AuthProvider):
    desc = 'ZJU Unified Identity Authentication'

    def __init__(self, session):
        super().__init__()
        self.session = session

    def get_pub_key(self):
        pub = self.session.get(PUB_KEY_URL).json()
        n = int(pub['modulus'], base=16)
        e = int(pub['exponent'], base=16)
        return rsa.PublicKey(n, e)

    def login(self, username: str, password: str):
        res = self.session.get(LOGIN_URL).text
        soup = BeautifulSoup(res, 'lxml')
        execution = soup.findAll('input', attrs={'name': 'execution'})[0]['value']

        pub_key = self.get_pub_key()
        encrypted_pass = rsa_encrypt(password.encode(), pub_key).hex()

        form = {
            'username': username,
            'password': encrypted_pass,
            'authcode': '',
            'execution': execution,
            '_eventId': 'submit'
        }
        self.session.post(LOGIN_URL, data=form)
        cookies = self.session.cookies
        if '_pc0' not in cookies:
            raise AuthError()
        return self.session



from pathlib import Path

from tqdm import tqdm

from tronclass_cli.command import Command
from tronclass_cli.middleware.api import ApiMiddleware


class ActivitiesDownloadCommand(Command):
    name = 'activities.download'
    middleware_classes = [ApiMiddleware]

    def _init_parser(self):
        self._parser.add_argument('reference_id', help='reference id of the file')
        self._parser.add_argument('output_file', type=Path, help='path to the output file')
        self._parser.add_argument('--preview', action='store_true', help='download the preview file')

    def _exec(self, args):
        res = self._ctx.api.get_document(args.reference_id, args.preview)
        total_size = int(res.headers.get('content-length', 0))
        block_size = 1024
        progress_bar = tqdm(total=total_size, unit='iB', unit_scale=True)
        args.output_file.parent.mkdir(parents=True, exist_ok=True)
        with open(args.output_file, 'wb') as fs:
            for data in res.iter_content(block_size):
                progress_bar.update(len(data))
                fs.write(data)
        progress_bar.close()
        if total_size != 0 and progress_bar.n != total_size:
            raise IOError('file size mismatches')



import json

from pathlib import Path


class Config:
    def __init__(self, data: dict):
        self._data = data

    def __getitem__(self, item):
        return self._data[item]

    def __setitem__(self, key, value):
        self._data[key] = value

    def get_section(self, name):
        return Config(self._data.get(name, {}))

    def get(self, name: str, default=None, required=False, value_type: type = None, type_converter=None):
        res = self._data.get(name, default)
        if required and res is None:
            raise KeyError(f'Config item {name} is required but missing.')
        if value_type is not None:
            try:
                if not isinstance(res, value_type):
                    res = type_converter(res)
            except Exception:
                raise ValueError(f'Config item {name} should be {type.__name__}.')
        return res

    def to_dict(self):
        return self._data


def load_config(path):
    path = Path(path)
    config_dict = {}
    if path.exists():
        with open(path) as fs:
            config_dict = json.load(fs)
    return Config(config_dict)



from datetime import timedelta
from getpass import getpass

import keyring
from requests import Session

from tronclass_cli.api.auth.providers import get_auth_provider, get_all_auth_providers
from tronclass_cli.middleware import Middleware
from tronclass_cli.middleware.cache import CacheMiddleware
from tronclass_cli.utils import interact

SERVICE_NAME = 'tronclass'
SESSION_LIFETIME = timedelta(hours=1)
DEFAULT_UA = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.190 Safari/537.36'


def try_get_password(service_name, username):
    try:
        return keyring.get_password(service_name, username)
    except:
        return None


def try_set_password(service_name, username, password):
    try:
        keyring.set_password(service_name, username, password)
        return True
    except:
        return False


class SessionMiddleware(Middleware):
    name = 'session'
    middleware_classes = [CacheMiddleware]

    def _init_parser(self):
        group = self._parser.add_argument_group(self.name)
        group.add_argument('--username', help='username for authentication')
        group.add_argument('--auth-provider', default='zju',
                           help=f'authentication provider, available providers: {", ".join(get_all_auth_providers().keys())}')
        group.add_argument('--no-save-credentials', dest='save_credentials', action='store_false',
                           help='do not save the credentials')
        group.add_argument('--user-agent', default=DEFAULT_UA, help='user agent sent with requests')

    def _exec(self, args):
        username = args.username or interact.prompt_input('Username')
        self._ctx.username = username
        self._ctx.session = self._ctx.cache.get(f'session.{username}')
        if self._ctx.session is not None:
            return
        password = try_get_password(SERVICE_NAME, username) or getpass()
        if args.save_credentials:
            try_set_password(SERVICE_NAME, username, password)

        session = Session()
        session.headers['User-Agent'] = args.user_agent
        auth_provider = get_auth_provider(args.auth_provider)(session)
        self._ctx.session = auth_provider.login(username, password)
        self._ctx.cache.set(f'session.{username}', self._ctx.session, SESSION_LIFETIME)



import tronclass_cli
from tronclass_cli.command import Command


class RootCommand(Command):
    name = 'root'

    def _init_parser(self):
        self._parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + tronclass_cli.__version__)

    def _exec(self, args):
        self._parser.print_help()

    def parse_args(self, *args, **kwargs):
        return self._parser.parse_args(*args, **kwargs)



from tronclass_cli.api import Api
from tronclass_cli.middleware import Middleware
from tronclass_cli.middleware.session import SessionMiddleware

api_urls = {
    'zju': 'https://courses.zju.edu.cn'
}


class ApiMiddleware(Middleware):
    name = 'api'
    middleware_classes = [SessionMiddleware]

    def _init_parser(self):
        group = self._parser.add_argument_group(self.name)
        group.add_argument('--api-url', default='zju',
                           help=f'base url of TronClass API, supported shortcuts: {", ".join(api_urls.keys())}')

    def _exec(self, args):
        api_url = api_urls.get(args.api_url, args.api_url)
        self._ctx.api = Api(api_url, self._ctx.username, self._ctx.cache, self._ctx.session)



from yaml import dump, CDumper as Dumper

from tronclass_cli.command import Command
from tronclass_cli.middleware.api import ApiMiddleware
from tronclass_cli.utils import nested_dict_select


class ActivitiesViewCommand(Command):
    name = 'activities.view'
    middleware_classes = [ApiMiddleware]

    def _init_parser(self):
        self._parser.add_argument('activity_id', help='activity id')
        default_fields = 'id,title,type,data,deadline,uploads'
        self._parser.add_argument('--fields', default=default_fields,
                                  help=f'fields to display, default fields: {default_fields}')

    def _exec(self, args):
        fields = args.fields.split(',')
        activity = self._ctx.api.get_activity(args.activity_id)
        print(dump(nested_dict_select(activity, fields), Dumper=Dumper, allow_unicode=True, sort_keys=False))



from tronclass_cli.command import Command
from tronclass_cli.middleware.api import ApiMiddleware
from tronclass_cli.middleware.table import TableMiddleware


class TodoCommand(Command):
    name = 'todo'
    middleware_classes = [ApiMiddleware, TableMiddleware]

    def _init_parser(self):
        default_fields = 'course_id,course_name,end_time,id,title,type'
        self._parser.add_argument('--fields', default=default_fields,
                                  help=f'fields to display, default fields: {default_fields}')

    def _exec(self, args):
        fields = args.fields.split(',')
        todo = list(self._ctx.api.get_todo())
        if len(todo) == 0:
            print('No courses.')
        else:
            self._ctx.print_table(todo, fields)



from tronclass_cli.middleware import Middleware


class Command(Middleware):
    name = None

    def __init__(self, parser, ctx):
        super().__init__(parser, ctx)
        self._sub_parsers = None

    def add_sub_command(self, name: str, sub_command_class, **kwargs):
        if self._sub_parsers is None:
            self._sub_parsers = self._parser.add_subparsers(title='subcommands')
        sub_parser = self._sub_parsers.add_parser(name, **kwargs)
        obj = sub_command_class(sub_parser, self._ctx)
        return obj



from dateutil.parser import isoparse


def iter_select_where(iter, predicate):
    for it in iter:
        if predicate(it):
            return it
    return None


def dict_select(dict, keys):
    return {k: dict.get(k) for k in keys}


def nested_dict_select(dict, keys):
    res = {}
    for key in keys:
        dst = res
        src = dict
        *layers, last = key.split('.')
        for k in layers:
            dst = dst.setdefault(k, {})
            src = src.setdefault(k, {})
        dst[last] = src.get(last)
    return res


def flatten_dict(d, separator='.'):
    out = {}
    for key, val in d.items():
        if isinstance(val, dict):
            val = [val]
        if isinstance(val, list):
            for child in val:
                if isinstance(child, dict):
                    deeper = flatten_dict(child)
                    for key2, val2 in deeper.items():
                        out.setdefault(key + separator + key2, []).append(val2)
                else:
                    out.setdefault(key, []).append(child)
        else:
            out[key] = val
    return out


def process_table_data(data, key_filter):
    data = flatten_dict(data)
    data = dict_select(data, key_filter)

    def process_value(value):
        if isinstance(value, list):
            return ', '.join([str(x) for x in value])
        try:
            date = isoparse(value)
            return date.astimezone().strftime('%c')
        except:
            return value
        return value

    return {k: process_value(v) for k, v in data.items()}


def unflatten_fields(flatten_fields):
    fields = {}
    for field in flatten_fields:
        cur = fields
        for layer in field.split('.'):
            cur = cur.setdefault(layer, {})

    def visit(d):
        return ','.join([k if len(v) == 0 else f'{k}({visit(v)})' for k, v in d.items()])

    return visit(fields)



from tronclass_cli.command import Command
from tronclass_cli.middleware.api import ApiMiddleware
from tronclass_cli.middleware.table import TableMiddleware
from tronclass_cli.utils import unflatten_fields


class ActivitiesListCommand(Command):
    name = 'activities.list'
    middleware_classes = [ApiMiddleware, TableMiddleware]

    def _init_parser(self):
        self._parser.add_argument('course_id', help='course id')
        default_fields = 'id,title,type'
        self._parser.add_argument('--fields', default=default_fields,
                                  help=f'fields to display, default fields: {default_fields}')

    def _exec(self, args):
        fields = args.fields.split(',')
        activities = list(self._ctx.api.get_activities(args.course_id, fields=unflatten_fields(fields)))
        if len(activities) == 0:
            print('No activities.')
        else:
            self._ctx.print_table(activities, fields)



import setuptools

import tronclass_cli

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="tronclass-cli",
    version=tronclass_cli.__version__,
    author="Howyoung Zhou",
    author_email="howyoungzhou@yahoo.com",
    description="Command-line interface for Tronclass.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/HowyoungZhou/tronclass-cli",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "tcc = tronclass_cli.__main__:main"
        ]
    },
    python_requires='>=3.5',
)



import shelve
from datetime import datetime


class CachedItem:
    def __init__(self, value, expire_time):
        self.value = value
        self.expire_time = expire_time


class Cache:
    def __init__(self, cache_file, force_update=False):
        self._shelve = shelve.open(str(cache_file))
        self._force_update = force_update

    def get(self, key, default=None):
        if self._force_update:
            return default
        item = self._shelve.get(key)
        if not item:
            return default
        if item.expire_time is None or item.expire_time >= datetime.now():
            return item.value
        return default

    def set(self, key, value, lifetime=None):
        self._shelve[key] = CachedItem(value, datetime.now() + lifetime if lifetime else None)

    def close(self):
        self._shelve.close()



import inspect
import json
from datetime import timedelta
from types import GeneratorType
from urllib.parse import urljoin

from bs4 import BeautifulSoup
from requests import Session


def cached(key, lifetime=None):
    def cache_decorator(func):
        def cached_func(*args, **kwargs):
            call_args = inspect.getcallargs(func, *args, **kwargs)
            self = call_args['self']
            cache_key = f'api.{self._username}.{key.format(**call_args)}'
            value = self._cache.get(cache_key)
            if value is not None:
                return value
            value = func(*args, **kwargs)
            if isinstance(value, GeneratorType):
                value = list(value)
            self._cache.set(cache_key, value, lifetime)
            return value

        return cached_func

    return cache_decorator


class Api:
    def __init__(self, base_url, username, cache, session: Session):
        self._base_url = base_url
        self._username = username
        self.session = session
        self._cache = cache

    def _get_api_url(self, path):
        return urljoin(self._base_url, path)

    def _api_call(self, path, method='GET', **kwargs):
        kwargs.setdefault('allow_redirects', True)
        return self.session.request(method, self._get_api_url(path), **kwargs)

    @cached('todo', timedelta(hours=6))
    def get_todo(self):
        return self._api_call('api/todos').json()['todo_list']

    @cached('user_id')
    def get_user_id(self):
        html = self._api_call('user/index').content
        soup = BeautifulSoup(html, 'html.parser')
        return soup.find(id='userId').get('value')

    def _get_pages(self, path, params, data_key, page_size=20):
        page = 1
        while True:
            api_params = {
                'page': page,
                'page_size': page_size,
                **params
            }
            res = self._api_call(path, params=api_params)
            res.raise_for_status()
            data = res.json()
            yield from data[data_key]

            if page >= data['pages']:
                break
            page += 1

    @cached('courses.{fields}.{conditions}', timedelta(days=1))
    def get_courses(self, conditions={}, fields='id,name'):
        user_id = self.get_user_id()
        params = {
            'conditions': json.dumps(conditions),
            'fields': fields,
        }
        return self._get_pages(f'api/users/{user_id}/courses', params, 'courses')

    @cached('homework.{course_id}.{conditions}', timedelta(hours=6))
    def get_homework(self, course_id, conditions={}):
        params = {
            'conditions': json.dumps(conditions),
        }
        return self._get_pages(f'api/courses/{course_id}/homework-activities', params, 'homework_activities')

    @cached('activities.{course_id}.{fields}', timedelta(hours=6))
    def get_activities(self, course_id, fields=''):
        params = {
            'fields': fields,
        } if fields != '' else None
        res = self._api_call(f'api/courses/{course_id}/activities', params=params)
        res.raise_for_status()
        return res.json()['activities']

    @cached('activity.{activity_id}', timedelta(hours=6))
    def get_activity(self, activity_id):
        res = self._api_call(f'api/activities/{activity_id}')
        res.raise_for_status()
        return res.json()

    def get_document(self, ref_id, preview=False):
        res = self._api_call(f'api/uploads/reference/document/{ref_id}/url',
                             params={'preview': str(preview).lower()})
        res.raise_for_status()
        url = res.json()['url']
        return self.session.get(url, stream=True)

    def post_uploads(self, name, size, parent_type=None, parent_id=0, is_scorm=False, is_wmpkg=False):
        data = {
            "name": name,
            "size": size,
            "parent_type": parent_type,
            "parent_id": parent_id,
            "is_scorm": is_scorm,
            "is_wmpkg": is_wmpkg
        }
        res = self._api_call(f'api/uploads', 'POST', json=data)
        res.raise_for_status()
        return res.json()

    def post_submissions(self, activity_id, uploads, slides=[], comment='', is_draft=False):
        data = {
            "comment": comment,
            "uploads": uploads,
            "slides": slides,
            "is_draft": is_draft
        }
        res = self._api_call(f'api/course/activities/{activity_id}/submissions', 'POST', json=data)
        res.raise_for_status()
        return res.json()



import os
from argparse import ArgumentParser
from pathlib import Path

from tronclass_cli.middleware.config import load_config


class Context:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def __eq__(self, other):
        if not isinstance(other, Context):
            return NotImplemented
        return vars(self) == vars(other)

    def __contains__(self, key):
        return key in self.__dict__

    def __repr__(self):
        return f'{self.__class__.__qualname__}({", ".join([f"{k} = {v}" for k, v in self.__dict__.items()])})'


class Middleware:
    name = None
    middleware_classes = []
    default_root = Path.home() / '.tronclass'
    config_path = Path(os.getenv('TRONCLASS_CLI_CONFIG_FILE') or default_root / 'config.json')
    config = load_config(config_path)

    def __init__(self, parser: ArgumentParser, ctx: Context):
        self._parser = parser
        self._ctx = ctx
        self._middlewares = [middleware_class(parser, ctx) for middleware_class in self.middleware_classes]
        self.init_parser()

    def init_parser(self):
        self._init_parser()
        self._parser.set_defaults(__middleware=self, **self.config.get_section(self.name).to_dict())

    def _init_parser(self):
        pass

    def exec(self, args):
        for middleware in self._middlewares:
            middleware.exec(args)
        self._exec(args)

    def _exec(self, args):
        pass

    def dispose(self):
        self._dispose()
        for middleware in reversed(self._middlewares):
            middleware.dispose()

    def _dispose(self):
        pass
